"use strict";

CodeMirror.defineMode("boethius", function (config) {
    var COMMENT = "comment";
    var STRING = "string";
    var STRING2 = "string-2";
    var NUMBER = "number";
    var VAR = "variable";
    var VAR2 = "variable-2";
    var VAR3 = "variable-3";
    var DEF = "def";
    var KEYWORD = "keyword";
    var OPERATOR = "operator";
    var ATOM = "atom";
    var META = "meta";
    var TAG = "tag";
    var ATTRIBUTE = "attribute";
    var QUALIFIER = "qualifier";
    var PROPERTY = "property";
    var BUILTIN = "builtin";

    var lexRules = {
        WHITESPACE: /\s+/,
        COMMENT: /\;.*/,
        BARLINE: /\|/,
        LPAREN: /\(/,
        RPAREN: /\)/,
        OPENBRKT: /\</,
        CLOSEBRKT: /\>/,
        FWDSLASH: /\//,
        EQUALS: /\=/,
        PITCH: /[a-gA-G][b|#]{0,2}[\d]+/,
        REST: /r/,
        DOTS: /\.+/,
        BOOL: /true|false/,
        INTEGER: /[0-9]+/,
        CSYM: /csym/,
        IDENTIFIER: /[a-zA-Z][a-zA-Z0-9]*/,
        INVALID: /./
    };

    return {
        token: function token(stream, state) {
            // ignore whitespace
            stream.eatSpace();

            if (stream.match(lexRules.COMMENT)) {
                stream.skipToEnd();
                return COMMENT;
            }

            if (stream.match(lexRules.BARLINE)) {
                return OPERATOR;
            }

            if (stream.match(lexRules.EQUALS)) {
                return OPERATOR;
            }

            if (stream.match(lexRules.PITCH)) {
                return ATOM;
            }

            if (stream.match(lexRules.REST)) {
                var w = stream.match(lexRules.WHITESPACE, false);
                var s = stream.match(lexRules.FWDSLASH, false);
                if (w || s || stream.eol()) return ATOM;
            }

            if (stream.match(lexRules.IDENTIFIER)) {
                return STRING;
            }

            if (stream.match(lexRules.INTEGER)) {
                return NUMBER;
            }

            // advance stream if nothing matches.
            stream.next();

            return null;
        }
    };
});
