CodeMirror.defineMode("boethius", config => {
    const COMMENT = "comment";
    const STRING = "string";
    const STRING2 = "string-2";
    const NUMBER = "number";
    const VAR = "variable";
    const VAR2 = "variable-2";
    const VAR3 = "variable-3";
    const DEF = "def";
    const KEYWORD = "keyword";
    const OPERATOR = "operator";
    const ATOM = "atom";
    const META = "meta";
    const TAG = "tag";
    const ATTRIBUTE = "attribute";
    const QUALIFIER = "qualifier";
    const PROPERTY = "property";
    const BUILTIN = "builtin";

    const lexRules = {
        WHITESPACE: /\s+/,
        COMMENT: /\;.*/,
        BARLINE: /\|/,
        LPAREN: /\(/,
        RPAREN: /\)/,
        OPENBRKT: /\</,
        CLOSEBRKT: /\>/,
        FWDSLASH: /\//,
        EQUALS: /\=/,
        PITCH: /[a-gA-G][b|#]{0,2}[\d]+/,
        REST: /r/,
        DOTS: /\.+/,
        BOOL: /true|false/,
        INTEGER: /[0-9]+/,
        CSYM: /csym/,
        IDENTIFIER: /[a-zA-Z][a-zA-Z0-9]*/,
        INVALID: /./
    };

    return {
        token (stream, state) {
            // ignore whitespace
            stream.eatSpace();

            if (stream.match(lexRules.COMMENT)) {
                stream.skipToEnd();
                return COMMENT;
            }

            if (stream.match(lexRules.BARLINE)) {
                return OPERATOR;
            }

            if (stream.match(lexRules.EQUALS)) {
                return OPERATOR;
            }

            if (stream.match(lexRules.PITCH)) {
                return ATOM;
            }

            if (stream.match(lexRules.REST)) {
                const w = stream.match(lexRules.WHITESPACE, false);
                const s = stream.match(lexRules.FWDSLASH, false);
                if (w || s || stream.eol()) return ATOM;
            }

            if (stream.match(lexRules.IDENTIFIER)) {
                return STRING;
            }

            if (stream.match(lexRules.INTEGER)) {
                return NUMBER;
            }

            // advance stream if nothing matches.
            stream.next();

            return null;
        }
    };
});
